"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
Object.defineProperty(exports, "__esModule", { value: true });
var version = require('../package.json').version;
require("font-awesome/css/font-awesome.css");
require("@phosphor/widgets/style/index.css");
require("@jupyter-widgets/controls/css/widgets.built.css");
/**
 * Load a package using requirejs and return a promise
 *
 * @param pkg Package name or names to load
 */
var requirePromise = function (pkg) {
    return new Promise(function (resolve, reject) {
        var require = window.require;
        if (require === undefined) {
            reject("Requirejs is needed to run the Jupyter Widgets html manager");
        }
        else {
            require(pkg, resolve, reject);
        }
    });
};
// Element.prototype.matches polyfill for IE
// See https://developer.mozilla.org/en-US/docs/Web/API/Element/matches
if (!Element.prototype.matches) {
    Element.prototype.matches =
        Element.prototype.msMatchesSelector ||
            Element.prototype.webkitMatchesSelector;
}
// Load json schema validator
var Ajv = require('ajv');
var widget_state_schema = require('@jupyter-widgets/schema').v2.state;
var widget_view_schema = require('@jupyter-widgets/schema').v2.view;
var ajv = new Ajv();
var model_validate = ajv.compile(widget_state_schema);
var view_validate = ajv.compile(widget_view_schema);
function loadManager() {
    var toLoad = ['base.js', 'controls.js', 'index.js'];
    return requirePromise(toLoad.map(function (f) { return "https://unpkg.com/@jupyter-widgets/html-manager@" + version + "/dist/" + f; }));
}
exports.loadManager = loadManager;
;
// `LoadInlineWidget` is the main function called on load of the web page. All
// it does is ensure requirejs is on the page and call `renderInlineWidgets`
function loadInlineWidgets(event) {
    loadManager().then(function () {
        renderInlineWidgets(event);
    });
}
// `renderInlineWidget` will call `renderManager(element, tag)` on each <script>
// tag of mime type
//     "application/vnd.jupyter.widget-state+json"
// contained in `event.element`.
function renderInlineWidgets(event) {
    var element = event.target || document;
    var tags = element.querySelectorAll('script[type="application/vnd.jupyter.widget-state+json"]');
    for (var i = 0; i != tags.length; ++i) {
        renderManager(element, tags[i]);
    }
}
exports.renderInlineWidgets = renderInlineWidgets;
// Function `renderManager(element, tag)` creates a widget embed manager for the
// specified <script> tag (which must have mime type)
//     "application/vnd.jupyter.widget-state+json".
// Then it performs a look up of all script tags under the specified DOM
// element with the mime type
//     "application/vnd.jupyter.widget-view+json".
// For each one of these <script> tags, if the contained json specifies a model id
// known to the aforementioned manager, it is replaced with a view of the model.
//
// Besides, if the view script tag has an <img> sibling DOM node with class `jupyter-widget`,
// the <img> tag is deleted.
function renderManager(element, tag) {
    window.require(['@jupyter-widgets/html-manager'], function (htmlmanager) {
        var widgetStateObject = JSON.parse(tag.innerHTML);
        var valid = model_validate(widgetStateObject);
        if (!valid) {
            console.log(model_validate.errors);
        }
        var manager = new htmlmanager.HTMLManager();
        manager.set_state(widgetStateObject).then(function (models) {
            var tags = element.querySelectorAll('script[type="application/vnd.jupyter.widget-view+json"]');
            var _loop_1 = function (i) {
                // TODO: validate view schema
                var viewtag = tags[i];
                var widgetViewObject = JSON.parse(viewtag.innerHTML);
                var valid_1 = view_validate(widgetViewObject);
                if (!valid_1) {
                    console.error('View state has errors.', view_validate.errors);
                }
                var model_id = widgetViewObject.model_id;
                // should use .find, but IE doesn't support .find
                var model = models.filter(function (item) {
                    return item.model_id == model_id;
                })[0];
                if (model !== undefined) {
                    if (viewtag.previousElementSibling &&
                        viewtag.previousElementSibling.matches('img.jupyter-widget')) {
                        viewtag.parentElement.removeChild(viewtag.previousElementSibling);
                    }
                    var widgetTag = document.createElement('div');
                    widgetTag.className = 'widget-subarea';
                    viewtag.parentElement.insertBefore(widgetTag, viewtag);
                    manager.display_model(undefined, model, { el: widgetTag });
                }
            };
            for (var i = 0; i != tags.length; ++i) {
                _loop_1(i);
            }
        });
    });
}
// Prevent the embedder to be defined twice.
if (!window._jupyter_widget_embedder) {
    window._jupyter_widget_embedder = true;
    window.addEventListener('load', loadInlineWidgets);
}
